@@ -34,7 +34,10 @@ static void init_buttons();
 static void init_text();
 
 double max_fix[MAXBASES] = {
-    6.871947674e+10, 3.245185537e+32, 1.000000000e+36, 2.230074520e+43
+    1.298074214e+33,    /* Binary. */
+    2.037035976e+90,    /* Octal. */
+    1.000000000e+100,   /* Decimal */
+    2.582249878e+120    /* Hexadecimal. */
 };
 
 char *base_str[]  = {          /* Strings for each base value. */
@@ -128,6 +131,8 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
    func_char
    menutype
    func
+   symname
+   flags
  */
 
 /* Row 1. */
@@ -141,7 +146,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },
 {
     N_("8"),
@@ -242,7 +247,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },
 {
     N_("5"),
@@ -254,7 +259,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },
 {
     N_("6"),
@@ -266,7 +271,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },
 {
     /* Note to translators: this is a multiplication sign (*) */
@@ -296,11 +301,11 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
 {
     /* Note to translators: this is a plus-minus sign (+/-) */
     N_("\302\261"),
-    N_("Change sign [c]"),
+    N_("Change sign [C]"),
     NULL,
-    { 0,     0 },
-    { GDK_c, 0 },
-    'c',
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_C,          0 },
+    'C',
     M_NONE,
     do_immed, 
     N_("Chs"),
@@ -328,7 +333,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_STO,
     do_pending,
     NULL,
-    none
+    pending
 },
 
 /* Row 3. */
@@ -342,7 +347,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | bin_set | oct_set
 },
 {
     N_("2"),
@@ -354,7 +359,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },    
 {     
     N_("3"),
@@ -366,7 +371,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | oct_set
 },
 {
     /* Note to translators: this is a minus sign (-) */
@@ -428,7 +433,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_RCL,
     do_pending,
     NULL,
-    none
+    pending
 },
 
 /* Row 4. */
@@ -442,7 +447,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_NONE,
     do_number,
     NULL,
-    number
+    number | bin_set | oct_set
 },
 {    
     N_("."),
@@ -527,7 +532,7 @@ struct button a_buttons[A_NOBUTTONS] = {   /* Advanced mode button values. */
     M_EXCH,
     do_pending,
     NULL,
-    none
+    pending
 },
 };
 
@@ -559,11 +564,11 @@ struct button f_buttons[F_NOBUTTONS] = {   /* Financial mode button values. */
 },
 {
     N_("Ddb"),
-    N_("Double-declining depreciation [d]"),
+    N_("Double-declining depreciation [D]"),
     NULL,
-    { 0,     0 },
-    { GDK_d, 0 },
-    'd',
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_D,          0 },
+    'D',
     M_NONE,
     do_business,
     NULL,
@@ -765,7 +770,7 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     M_LSHF,
     do_pending,
     NULL,
-    none
+    pending
 },
 {
     N_(">"),
@@ -777,7 +782,7 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     M_RSHF,   
     do_pending,
     NULL,
-    none
+    pending
 },            
 {             
     N_("&amp;16"),
@@ -841,15 +846,15 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 },
 {
     N_("Acc"),
-    N_("Set accuracy from 0 to 9 numeric places [a]"),
+    N_("Set accuracy from 0 to 9 numeric places [A]"),
     N_("Accuracy"),
-    { 0,     0 },
-    { GDK_a, 0 },
-    'a',
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_A,          0 },
+    'A',
     M_ACC,
     do_pending,
     NULL,
-    none
+    pending
 },
 
 /* Row 2. */
@@ -863,27 +868,27 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     M_CON,            
     do_pending,
     NULL,
-    none
+    pending
 },
 {
     N_("Fun"),
-    N_("User-defined functions [f]"),
+    N_("User-defined functions [F]"),
     NULL,
-    { 0,     0 },
-    { GDK_f, 0 },
-    'f',
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_F,          0 },
+    'F',
     M_FUN,
     do_pending,
     NULL,
-    none
+    pending
 },
 {
     N_("Exp"),
-    N_("Enter an exponential number [e]"),
+    N_("Enter an exponential number [E]"),
     N_("Exponential"),
-    { 0,     0 },
-    { GDK_e, 0 },
-    'e',
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_E,          0 },
+    'E',
     M_NONE,
     do_expno,
     N_("e"),
@@ -953,39 +958,39 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 /* Row 3. */
 {
     N_("D"),
-    N_("Hexadecimal digit D"),
+    N_("Hexadecimal digit D [d]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_D,          0 },
-    'D',
+    { 0,     0 },
+    { GDK_d, 0 },
+    'd',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },
 {
     N_("E"),
-    N_("Hexadecimal digit E"),
+    N_("Hexadecimal digit E [e]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_E,          0 },
-    'E',
+    { 0,     0 },
+    { GDK_e, 0 },
+    'e',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },
 {
     N_("F"),
-    N_("Hexadecimal digit F"),
+    N_("Hexadecimal digit F [f]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_F,          0 },
-    'F',
+    { 0,     0 },
+    { GDK_f, 0 },
+    'f',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },
 {
     N_("Cos"),
@@ -1051,42 +1056,42 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 /* Row 4. */
 {
     N_("A"),
-    N_("Hexadecimal digit A"),
+    N_("Hexadecimal digit A [a]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_A,          0 },
-    'A',
+    { 0,     0 },
+    { GDK_a, 0 },
+    'a',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },
 {
     N_("B"),
-    N_("Hexadecimal digit B"),
+    N_("Hexadecimal digit B [b]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_B,          0 },
-    'B',
+    { 0,     0 },
+    { GDK_b, 0 },
+    'b',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },    
 {     
     N_("C"),
-    N_("Hexadecimal digit C"),
+    N_("Hexadecimal digit C [c]"),
     NULL,
-    { GDK_SHIFT_MASK, 0 },
-    { GDK_C,          0 },
-    'C',
+    { 0,     0 },
+    { GDK_c, 0 },
+    'c',
     M_NONE,
     do_number,
     NULL,
-    number
+    number | hex
 },
 {
-    N_("Or"),
+    N_("OR"),
     N_("Bitwise OR"),
     "bitwise OR [!]",
     { GDK_SHIFT_MASK, 0 },
@@ -1094,11 +1099,11 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     '|',
     M_NONE,
     do_calc,
-    N_(" Or "),
+    N_(" OR "),
     binop
 },
 {
-    N_("And"),
+    N_("AND"),
     N_("Bitwise AND [&]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
@@ -1106,11 +1111,11 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     '&',
     M_NONE,
     do_calc,
-    N_(" And "),
+    N_(" AND "),
     binop
 },       
 {        
-    N_("Not"),
+    N_("NOT"),
     N_("Bitwise NOT [~]"),
     NULL,
     { GDK_SHIFT_MASK, 0 },
@@ -1122,7 +1127,7 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     unop | immediate
 },
 {
-    N_("Xor"),
+    N_("XOR"),
     N_("Bitwise XOR [x]"),
     NULL,
     { 0, 0 },   
@@ -1130,11 +1135,11 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     'x',
     M_NONE,
     do_calc,
-    N_(" Xor "),
+    N_(" XOR "),
     binop
 },
 {
-    N_("Xnor"),
+    N_("XNOR"),
     N_("Bitwise XNOR [n]"),
     NULL,
     { 0, 0 },
@@ -1142,7 +1147,7 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
     'n',
     M_NONE,
     do_calc,
-    N_(" Xnor "),
+    N_(" XNOR "),
     binop
 },
 };
