@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.87 2006/10/30 18:12:46 richb Exp $
+/*  $Header$
  *
- *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -61,6 +61,39 @@ gc_strdup(char *str)
     return(dup);
 }
 
+void
+make_exp(char *number, int t[MP_SIZE])
+{
+    int i;
+    char *a = NULL;
+    char *b = NULL;
+
+    int MP_a[MP_SIZE];
+    int MP_b[MP_SIZE];
+
+    assert(number);
+    a = gc_strdup(number);
+    assert(a);
+
+    for (i = 0; !((a[i] == 'e') || (a[i] == 'E')); i++) {
+        assert(a[i]);
+    }
+    
+    a[i] = 0;
+    b = &a[i+2];
+
+    MPstr_to_num(a, v->base, MP_a);
+    MPstr_to_num(b, v->base, MP_b);
+    if (a[i+1] == '-') {
+        int MP_c[MP_SIZE];
+        mpneg(MP_b, MP_c);   
+        calc_xtimestenpowx(MP_a, MP_c, t);
+    } else {
+        calc_xtimestenpowx(MP_a, MP_b, t);
+    }
+
+    free(a);
+}
 void
 update_undo_redo_button_sensitivity(void)
 {
@@ -248,6 +281,20 @@ exp_append(char *text)
 }
 
 
+void
+exp_insert(char *text)
+{
+    // struct exprm_state *e;
+    // e = get_state();
+    // write_display(e->expression);
+    if (v->ghost_zero) {
+        write_display("");
+        v->ghost_zero = 0;
+    }
+    insert_to_cursor(text);
+    get_expr_from_display();
+}
+
 void 
 exp_del() 
 {
@@ -299,7 +346,7 @@ exp_del_char(char **expr, int amount)
 }
 
 
-static void
+void
 exp_replace(char *text)
 {
     struct exprm_state *e = get_state();
@@ -395,7 +442,7 @@ str_replace(char **str, char *from, char *to)
     for (i = 0; len-i >= flen; i++) {
         if (!strncasecmp(from, *str+i, flen)) {
             char *print;
-            int j = i+3;
+            int j = i+flen;
             char *prefix = malloc(i+1);
             char *postfix = malloc(len-j+1);
 
@@ -403,7 +450,7 @@ str_replace(char **str, char *from, char *to)
             memset(prefix, 0, i+1);
             memset(postfix, 0, len-j+1);
             memcpy(prefix, *str, i);
-            memcpy(postfix, *str+i+3, len-j);
+            memcpy(postfix, *str+i+flen, len-j);
 
             print = malloc(strlen(to)+i+len-j+1);
             SPRINTF(print, "%s%s%s", prefix, to, postfix);
@@ -446,6 +493,7 @@ trig_filter(char **func)
 void
 do_expression()
 {
+    int update_display = 0; // Update whole display. Looses cursor position.
     char *btext = NULL;
     struct exprm_state *e;
     int non_empty_expression;
@@ -467,6 +515,7 @@ do_expression()
 	if (!strcmp(e->expression, "Ans")) {
 	    if (e->button.flags & number) {
 		exp_del(); 
+                update_display = 1;
 	    }	
 	}
     } else {
@@ -475,7 +524,7 @@ do_expression()
 	    char *zero = NULL;
 	    do_zero(MP1);
 	    zero = make_number(MP1, v->base, FALSE);
-	    exp_append(zero);
+	    exp_insert(zero);
 	}
     }
 
@@ -485,8 +534,10 @@ do_expression()
 	    char buf[1024];
 	    snprintf(buf, 128, "%s(Ans)", btext);
 	    exp_replace(buf);
+            update_display = 1;
     } else if (e->button.flags & clear) {
         exp_del();
+        update_display = 1;
         set_error_state(FALSE);
         MPstr_to_num("0", DEC, e->ans);
     } else if (e->button.flags & regrcl) {
@@ -495,21 +546,42 @@ do_expression()
         int n = '0' +  i;
 
         snprintf(reg, 3, "R%c", n);
-        exp_append(reg);
+        exp_insert(reg);
     } else if (e->button.flags & con) {
         int *MPval = v->MPcon_vals[char_val(e->button.value[0])];
 
-        exp_append(make_number(MPval, v->base, FALSE));
+        exp_insert(make_number(MPval, v->base, FALSE));
     } else if (e->button.flags & bsp) {
         if (exp_has_postfix(e->expression, "Ans")) { 
             char *ans = make_number(e->ans, v->base, FALSE);   
+
             str_replace(&e->expression, "Ans", ans);
-        } 
-        exp_del_char(&e->expression, 1);
+        } else {
+            char reg[3];
+            int n = '0';
+            int i;
+
+            for (i = 0; i < 10; i++) {
+                snprintf(reg, 3, "R%c", n+i);
+                if (exp_has_postfix(e->expression, reg)) {
+                    int MP_reg[MP_SIZE];
+                    char *reg_val;
+
+                    do_rcl_reg(i, MP_reg);
+                    reg_val = make_number(MP_reg, v->base, FALSE);
+                    /* Remove "Rx" postfix. */
+                    exp_del_char(&e->expression, 2);
+                    exp_insert(reg_val);
+                }
+            }
+        }
+        delete_from_cursor();
     } else if (e->button.flags & neg) {
         exp_negate();
+        update_display = 1;
     } else if (e->button.flags & inv) {
         exp_inv();
+        update_display = 1;
     } else if (e->button.flags & enter) {
 	  if (e->expression) {
 	    if (strcmp(e->expression, "Ans")) {
@@ -519,6 +591,7 @@ do_expression()
 		  if (!ret) {
 			mpstr(MPval, e->ans);
 			exp_replace("Ans");
+                        update_display = 1;
 		  } else {
 			char *message = NULL;
 
@@ -535,6 +608,10 @@ do_expression()
 			        message = _("Invalid bitwise operation parameter(s)");
 			        break;
 
+			    case -PARSER_ERR_MODULUSOP:
+			        message = _("Invalid modulus operation parameter(s)");
+			        break;
+
 			    case -MPMATH_ERR:
 			        message = _("Math operation error");
 			        break;
@@ -548,20 +625,22 @@ do_expression()
 		  perform_undo();
 		  if (is_undo_step()) {
 			perform_undo();
+                        update_display = 1;
 		  }
 	    }
 	  }
     } else {
-	exp_append(btext);
+	exp_insert(btext);
 	if (e->button.flags & func) {
-	    exp_append("(");
+	    exp_insert("(");
 	}
     }
 
     free(btext);
     btext = NULL;
-    refresh_display();
-    return;
+    if (update_display) {
+        refresh_display();
+    }
 }
 
 
@@ -621,25 +700,25 @@ do_calc()      /* Perform arithmetic calculation and display result. */
     } else if (IS_KEY(v->cur_op, KEY_YTOX.value[0])) {    /* y^x */
         calc_xpowy(v->MPresult, v->MPdisp_val, v->MPresult);
 
-    } else if (IS_KEY(v->cur_op, KEY_AND.value[0])) {     /* And */
+    } else if (IS_KEY(v->cur_op, KEY_AND.value[0])) {     /* AND */
         mpcmd(v->MPresult, &dres);
         mpcmd(v->MPdisp_val, &dval);
         dres = setbool(ibool(dres) & ibool(dval));
         mpcdm(&dres, v->MPresult);
 
-    } else if (IS_KEY(v->cur_op, KEY_OR.value[0])) {      /* Or */
+    } else if (IS_KEY(v->cur_op, KEY_OR.value[0])) {      /* OR */
         mpcmd(v->MPresult, &dres);
         mpcmd(v->MPdisp_val, &dval);
         dres = setbool(ibool(dres) | ibool(dval));
         mpcdm(&dres, v->MPresult);
 
-    } else if (IS_KEY(v->cur_op, KEY_XOR.value[0])) {     /* Xor */
+    } else if (IS_KEY(v->cur_op, KEY_XOR.value[0])) {     /* XOR */
         mpcmd(v->MPresult, &dres);
         mpcmd(v->MPdisp_val, &dval);
         dres = setbool(ibool(dres) ^ ibool(dval));
         mpcdm(&dres, v->MPresult); 
 
-    } else if (IS_KEY(v->cur_op, KEY_XNOR.value[0])) {    /* Xnor */
+    } else if (IS_KEY(v->cur_op, KEY_XNOR.value[0])) {    /* XNOR */
         mpcmd(v->MPresult, &dres);
         mpcmd(v->MPdisp_val, &dval);
         dres = setbool(~ibool(dres) ^ ibool(dval));
@@ -774,7 +853,7 @@ do_base(enum base_type b)    /* Change the current base setting. */
 	if (ret) {
 	    update_statusbar(_("No sane value to convert"), 
 			     "gtk-dialog-error");
-	} else {
+	} else if (!v->ghost_zero) {
 	    mpstr(MP, e->ans);
 	    exp_replace("Ans");
 	    make_registers();
@@ -1033,7 +1112,7 @@ do_immedfunc(int s[MP_SIZE], int t[MP_SIZE])
     } else if (key_equal(v->current, KEY_SQRT)) {         /* Sqrt */
         mpstr(s, MP1);
         mpsqrt(MP1, t);
-    } else if (key_equal(v->current, KEY_NOT)) {          /* Not */
+    } else if (key_equal(v->current, KEY_NOT)) {          /* NOT */
         calc_not(t, s);
     } else if (key_equal(v->current, KEY_REC)) {          /* 1/x */
         calc_inv(s, t);
@@ -1155,7 +1234,7 @@ do_numtype(enum num_type n)   /* Set number display type. */
         if (ret) {
             update_statusbar(_("No sane value to convert"),
                              "gtk-dialog-error");
-        } else {
+        } else if (!v->ghost_zero) {
             mpstr(MP, e->ans);
             exp_replace("Ans");
             make_registers();
@@ -1460,8 +1539,8 @@ do_shift()     /* Perform bitwise shift on display value. */
             int n = char_val(e->button.value[0]);
             int ret = usable_num(MPval);
 
-            if (ret) {
-	        update_statusbar(_("No sane value to store"), 
+            if (ret || !is_integer(MPval)) {
+	        update_statusbar(_("No sane value to do bitwise shift"), 
                                  "gtk-dialog-error");
 	        return;
             } 
